<?php

namespace Annie\Stories\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Annie\Stories\Http\Requests\SuccessStoryRequest;
use Annie\Stories\Interfaces\SuccessStoryRepositoryInterface;
use Annie\Stories\Models\SuccessStory;

/**
 * Resource controller class for success_story.
 */
class SuccessStoryResourceController extends BaseController
{

    /**
     * Initialize success_story resource controller.
     *
     * @param type SuccessStoryRepositoryInterface $success_story
     *
     * @return null
     */
    public function __construct(SuccessStoryRepositoryInterface $success_story)
    {
        parent::__construct();
        $this->repository = $success_story;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Annie\Stories\Repositories\Criteria\SuccessStoryResourceCriteria::class);
    }

    /**
     * Display a list of success_story.
     *
     * @return Response
     */
    public function index(SuccessStoryRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Annie\Stories\Repositories\Presenter\SuccessStoryPresenter::class)
                ->$function();
        }

        $success_stories = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('stories::success_story.names'))
            ->view('stories::success_story.index', true)
            ->data(compact('success_stories', 'view'))
            ->output();
    }

    /**
     * Display success_story.
     *
     * @param Request $request
     * @param Model   $success_story
     *
     * @return Response
     */
    public function show(SuccessStoryRequest $request, SuccessStory $success_story)
    {

        if ($success_story->exists) {
            $view = 'stories::success_story.show';
        } else {
            $view = 'stories::success_story.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('stories::success_story.name'))
            ->data(compact('success_story'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new success_story.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SuccessStoryRequest $request)
    {

        $success_story = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('stories::success_story.name')) 
            ->view('stories::success_story.create', true) 
            ->data(compact('success_story'))
            ->output();
    }

    /**
     * Create new success_story.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SuccessStoryRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $success_story                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('stories::success_story.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('stories/success_story/' . $success_story->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/stories/success_story'))
                ->redirect();
        }

    }

    /**
     * Show success_story for editing.
     *
     * @param Request $request
     * @param Model   $success_story
     *
     * @return Response
     */
    public function edit(SuccessStoryRequest $request, SuccessStory $success_story)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('stories::success_story.name'))
            ->view('stories::success_story.edit', true)
            ->data(compact('success_story'))
            ->output();
    }

    /**
     * Update the success_story.
     *
     * @param Request $request
     * @param Model   $success_story
     *
     * @return Response
     */
    public function update(SuccessStoryRequest $request, SuccessStory $success_story)
    {
        try {
            $attributes = $request->all();

            $success_story->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('stories::success_story.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('stories/success_story/' . $success_story->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('stories/success_story/' . $success_story->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the success_story.
     *
     * @param Model   $success_story
     *
     * @return Response
     */
    public function destroy(SuccessStoryRequest $request, SuccessStory $success_story)
    {
        try {

            $success_story->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('stories::success_story.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('stories/success_story/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('stories/success_story/' . $success_story->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple success_story.
     *
     * @param Model   $success_story
     *
     * @return Response
     */
    public function delete(SuccessStoryRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('stories::success_story.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('stories/success_story'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/stories/success_story'))
                ->redirect();
        }

    }

    /**
     * Restore deleted success_stories.
     *
     * @param Model   $success_story
     *
     * @return Response
     */
    public function restore(SuccessStoryRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('stories::success_story.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/stories/success_story'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/stories/success_story/'))
                ->redirect();
        }

    }

}
